/*******************************************************************************
  *
  *  program     : MyString.java
  *  author      : Helge Janicke
  *  version     : 2000-10-17(001)
  *  (c)         : Helge Janicke
  *  email       : heljanic@agent.fho-emden.de
  *  description : Theoretische Informatik Praktikum Aufgabe 3.
  *				   bungen zur Rekursion.
  *
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic    your description here
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/

package dea;

import java.lang.*;
import java.io.*;


/** Klasse, welche die statischen Methoden enthlt, die in Aufgabe 3 implementiert
  * werden sollen.
  * @author (c) Helge Janicke
  * @version 2000-10-17(001)
  */
public class MyString {

	/** Zhler der die rekursiven Aufrufe mitzhlt.*/
	static int recCounter = 0;

	/** Standardkonstuktor */
	public MyString() {
	}

	/** Liefert den ersten Buchstaben.
	  * @param w Das Wort.
	  * @return Den ersten Buchstaben.
	  */
	public static char first(String w) {
		char rv = '\0';
		if (w.length() > 0) rv = w.charAt(0);
		return rv;
	}

	/** Liefert den Rest eines Wortes nach dem ersten Buchstaben.
	  * @param w Das Wort.
	  * @return Den Rest des Wortes.
	  */
	public static String rest(String w) {
		String rv = "";
		if (w.length() > 0) rv = w.substring(1);
		return rv;
	}

	/** Gibt an, ob ein Wort leer ist.
	  * @param w Das Wort.
	  * @return true, wenn w == leeres Wort, ansonsten false.
	  */
	public static boolean isEmpty(String w) {
		return (w.length() == 0);
	}

	/** Die Methode, die das rekursive Zhlen durchfhrt.
	  * @param w Das Wort.
	  * @param b der Buchstabe, der gezhlt werden soll.
	  * @return die Anzahl der gefundenen b's.
	  */
	private static int zaehleRekursiv(String w, char b) {
		int bcounter = 0;
		char firstChar;
		recCounter++;

		if (!isEmpty(w)) {
			firstChar = first(w);
			if (firstChar == b) bcounter=1;
			bcounter += zaehleRekursiv(rest(w), b);
		}

		recCounter--;
		return bcounter;
	}

	/** Der Aufruf, mit dem das rekursive Zhlen gestartet wird.
	  * @param w Das Wort.
	  * @param b Der Buchstabe, der gezhlt werden soll.
	  * @return Die Anzahl der gefundenen b's.
	  */
	public static int zaehle(String w, char b) {
		int rv;
		recCounter = 0;
		rv = zaehleRekursiv(w, b);
		if (recCounter != 0)
			System.err.println("ERROR:\n\tRekursionsfehler int zaehle(...), Stack nicht korrekt abgebaut");
		return rv;
	}

	/** Die Methode, die das rekursive Wiederholen durchfhrt.
	  * @param w Das Wort.
	  * @param n die Anzahl der Wiederholungen.
	  * @return Das potenzierte Wort.
	  */
	private static String expRekursiv(String w, int n) {
		recCounter++;
		if (n>1) {
			w += expRekursiv(w, n-1);
		}
		recCounter--;
		return w;
	}

	/** Der Aufruf, mit dem das rekursive Wiederholen gestartet wird.
	  * @param w Das Wort.
	  * @param n die Anzahl der Wiederholungen.
	  * @return Das potenzierte Wort.
	  */
	public static String exp(String w, int n) {
		String rv;
		recCounter = 0;
		if (n > 0) rv = expRekursiv(w, n);
		else rv = "";
		if (recCounter != 0)
			System.err.println("ERROR:\n\tRekursionsfehler in exp(...), Stack nicht korrekt abgebaut");
		return rv;
	}

	/** Die Methode, die das rekursive Spiegeln durchfhrt.
	  * @param w Das Wort.
	  * @return Das gespiegelte Wort.
	  */
	private static String reverseRekursiv(String w) {
		char firstChar;
		String rv = "";
		recCounter++;
		if (!isEmpty(w)) {
			firstChar = first(w);
			rv += reverseRekursiv(rest(w)) + firstChar;
		}
		recCounter--;
		return rv;
	}


	/** Der Aufruf, der das rekursive Spiegeln durchfhrt.
	  * @param w Das Wort.
	  * @return Das gespiegelte Wort.
	  */
	public static String reverse(String w) {
		String rv;
		recCounter = 0;
		rv = reverseRekursiv(w);
		if (recCounter != 0)
			System.err.println("ERROR:\n\tRekursionsfehler in reverse(...), Stack nicht korrekt abgebaut");
		return rv;
	}


	/** Hauptprogramm. */
	public static void main(String[] args) {
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		String word = "";
		String chr = "";
		String exp = "";
		char b;
		int n;
		boolean again = true;

		do {
			try {
				System.out.println("\nGeben Sie ein Wort ein.");
				System.out.print("w = ");
				word = in.readLine();
				System.out.println("\nGeben Sie ein Zeichen ein, sollten Sie mehr als ein Zeichen eingeben wird nur\n das erste Zeichen gewertet.");
				System.out.print("b = ");
				chr = in.readLine();
				System.out.println("\nGeben Sie eine ganze Zahl >=0 ein.");
				System.out.print("n = ");
				exp = in.readLine();
			} catch (IOException exc) {
				exc.printStackTrace();
			}

			try {
				b = chr.charAt(0);
				n = Integer.parseInt(exp);
				System.out.println("\n\nIhre Eingabe:\n");
				System.out.println("w       = " + word);
				System.out.println("b       = " + b);
				System.out.println("n       = " + n);
				System.out.println("------------------------------------------\n");
				System.out.println("Programmausgabe:\n");
				System.out.println("first   = " + MyString.first(word));
				System.out.println("rest    = " + MyString.rest(word));
				System.out.println("isEmpty = " + MyString.isEmpty(word));
				System.out.println("|w|b    = " + MyString.zaehle(word, b));
				System.out.println("(w)^n   = " + MyString.exp(word, n));
				System.out.println("reverse = " + MyString.reverse(word));
			} catch (NumberFormatException exc) {
				System.out.println("\n\nIhre Eingabe:");
				System.out.println("w       = " + word);
				System.out.println("b       = " + chr);
				System.out.println("n       = " + exp);
				System.out.println("...war fehlerhaft. Sie haben fuer n keine ganze Zahl eingegeben.\nBitte versuchen Sie es nocheinmal.");
			} catch (IndexOutOfBoundsException exc) {
				System.out.println("\n\nIhre Eingabe:");
				System.out.println("w       = " + word);
				System.out.println("b       = " + chr);
				System.out.println("n       = " + exp);
				System.out.println("...war fehlerhaft. Sie haben fuer b keinen Buchstaben eingegeben.\nBitte versuchen Sie es nocheinmal.");
			}
			System.out.print("\n\nNochmal (ja = irgentwas / nein = n)? ");
			try {
				if ((in.readLine()).equalsIgnoreCase("n")) again = false;
			} catch (IOException exc) {
				again = false;
			}
		} while (again);

	}

}





